from torch.utils.data import Dataset, DataLoader
import tensorflow as tf

class MyTorchDataset(Dataset):
    def __init__(self, data):
        self.data = data
    def __len__(self):
        return len(self.data)
    def __getitem__(self, idx):
        return self.data[idx]

def Torch_loader2TF_loader(torch_dataloader, batch_size, length=10e30):
  data = []
  for i, x in enumerate(torch_dataloader):
    data.append([tensor.numpy() for tensor in x])
    if i >= length*batch_size:
      break
  data = [list(x) for x in zip(*data)]
  return tf.data.Dataset.from_tensor_slices(tuple(data)).batch(batch_size)

def TF_loader2Torch_loader(tf_dataloader, batch_size, length=10e30):
  unbatch_tf_dataloader = tf_dataloader.unbatch()
  data = []
  for i, x in enumerate(unbatch_tf_dataloader):
    data.append(tuple(tensor.numpy() for tensor in x))
    if i >= length*batch_size:
      break
  return DataLoader(MyTorchDataset(data), batch_size=batch_size)


from .copy_weights import W_Torch2TF
from .covert_dataloader import Torch_loader2TF_loader, TF_loader2Torch_loader
import torch
import tensorflow as tf
from .trainer_checker import TrainerChecker
from .utils import SetZeroLearningRate

def test_trainer(
    tf_trainer, torch_trainer, 
    tf_model, torch_model,
    tf_optimizer, torch_optimizer,
    tf_train_loader = None, torch_train_loader = None,
    tf_test_loader = None, torch_test_loader = None,
    train_metrics = ['loss'], test_metrics = ['full'],
    batch_size = 16, loader_length=10,
    ):
  is_train = tf_train_loader or torch_train_loader
  is_test = tf_test_loader or torch_test_loader
  # Check dataloader
  if torch_train_loader: 
    tf_train_loader = Torch_loader2TF_loader(torch_train_loader, batch_size, loader_length)
  if tf_train_loader: 
    torch_train_loader = TF_loader2Torch_loader(tf_train_loader, batch_size, loader_length)
    tf_train_loader = tf_train_loader.unbatch().batch(batch_size)
  if torch_test_loader: 
    tf_test_loader = Torch_loader2TF_loader(torch_test_loader, batch_size, loader_length)
  if tf_test_loader: 
    torch_test_loader = TF_loader2Torch_loader(tf_test_loader, batch_size, loader_length)
    tf_test_loader = tf_test_loader.unbatch().batch(batch_size)
  print("Dataloader passed !!\n")
  # Copy weights
  W_Torch2TF(torch_model, tf_model)
  print("Copy weights TF2Torch passed !!\n")
  # To device
  if torch.cuda.is_available():
    torch_model.to('cuda')
  # SetZeroLearningRate
  SetZeroLearningRate(torch_optimizer, tf_optimizer)
  print("SetZeroLearningRate passed !!\n")
  # tf_train_loader = tf_train_loader.unbatch().batch(batch_size)
  trainer_checker = TrainerChecker(
    torch_trainer, tf_trainer, 
    train_metrics, test_metrics,
    loader_length, batch_size
    )
  if is_test:
    if not trainer_checker.chek_test_step(tf_test_loader, torch_test_loader):
      return False
  if is_train:
    if not trainer_checker.check_train_step(tf_train_loader, torch_train_loader):
      return False
  return True


import tensorflow as tf

def SetZeroLearningRate(torch_optimizer, tf_optimizer):
  new_lr = 0.
  for param_group in torch_optimizer.param_groups:
    param_group['lr'] = new_lr
  tf.keras.backend.set_value(tf_optimizer.learning_rate, new_lr)


from setuptools import setup, find_packages

setup(
    name='mltrainer-checker',
    version='0.15',
    description='A tool to validate machine learning models implemented on different machine learning platforms to ensure they are working correctly.',
    author='hiendang7613',
    author_email='dvhqnn1@gmail.com',
    url='https://github.com/hiendang7613/MLTrainerChecker',
    packages=find_packages(),
)
